Appendix A

Troubleshooting

 One of the main reasons for QEMM's and QRAM's existence is to help manage
your memory~dash~so that you can use all of the TSRs, device drivers, and
add-on hardware you want without using so much conventional memory that your
applications programs can no longer start up. However, with this power there
is also danger. That danger is that, now that your programs can fit into
memory together, there is some conflict between their functions which cause
one or both of them to not work. This incompatibility might, in extreme
cases, cause your system to hang.

This class of problems, (the incompatibility between two or more elements of
your system), is not new. But now that you have the ability to use so many
pieces of software at once, it becomes more likely to occur. For this reason,
you should know how to diagnose a problem should one crop up.  Your ability
to do this could save you hours of time on long-distance phone calls trying
to get tech support from one vendor after another.

To diagnose a problem when you have many pieces of software loaded, you must
determine whether one of your programs simply does not work at all, or
whether there is some incompatibility between two or more of the programs you
are using. The technique we suggest for isolating what is going wrong is to
create a "pure" environment for your program. A pure environment is one in
which any non-essential drivers or resident programs have been removed from
your CONFIG.SYS and AUTOEXEC.BAT files. In difficult cases, where alternate
hardware is available, creating a pure environment can even be extended to
removing non-essential hardware devices or replacing non-standard ones with
devices that are more common and known to be sound.

Here is a step-by-step procedure for troubleshooting and creating a pure
environment:

~Item~ Determine whether the problem might be something you have recently
changed. If the problem began when you installed something new into your
system, or changed the configuration of one of the elements of your system,
then uninstall the new element or restore the old configuration for it, and
see if the problem goes away.  If it does, you have already isolated one part
of the problem. If not, go on to the next step.

~Item~ Make a bootable floppy disk. If you normally boot from your hard disk,
make sure before you make any changes that you have the means to boot from a
floppy disk, and that you will be able to access and edit CONFIG.SYS or
AUTOEXEC.BAT on your hard disk, having booted from a floppy. If you are not
totally familiar with the operation of the drivers in your CONFIG.SYS, you
could find that removing one may make your system unable to boot. A bootable
floppy enables you to get started again. Make a dry run to be sure you can
boot from the floppy.

~Item~ Make copies of your current CONFIG.SYS and AUTOEXEC.BAT files.  Use
file names other than CONFIG.BAK and AUTOEXEC.BAK, as these tend to get
overwritten by text editors. These copies will give you a reference to what
you are about to change and provide a way to get the system back to the state
it was in before you started.

~Item~ Remove statements from these files by putting the word "REM" and a
space on the line in the file which loads the resident program or driver.  If
your DOS version is prior to 4.0, the use of REM in the CONFIG.SYS file will
cause an error message to be displayed on boot which says "Unrecognized
command in CONFIG.SYS", but this error message can be ignored. DOS will
bypass the loading of the driver, which is what we want at this point.

~Item~ Avoid removing statements from either file that are absolutely needed
for your system's basic functionality. If Hard disk drivers are needed to
boot your system, they should generally not be removed, while RAM disk
drivers or disk caches probably should be removed. It is not necessary to
remove PATH= or PROMPT= statements from your AUTOEXEC.BAT file as these are
almost always benign and removing them will simply make life difficult for
you. You should probably be more aggressive about removing other statements
in your AUTOEXEC.BAT. Do not assume that any utility is benign, just because
you have been using it for a long time. Remove it now~dash~you can add it
back and exonerate it later.

~Item~ Test the system now to see if a problem still occurs. If it does, even
in this "pure environment", then either your hardware is faulty, or the
program you are testing is fundamentally incompatible with your hardware. If
this is the case, have your hardware checked out. If it seems OK, contact the
maker of the hardware and software to see if they are aware of a solution. If
a problem does not occur, then go on to the next step.

~Item~ Add the statements to either file which pertain to the specific area
you feel could be the problem. For instance, if you are analyzing or
trouble-shooting in the memory area, you would want to enable your memory
manager at this point.

~Item~ Retest the system with this new addition. If the driver you thought
might be the problem proves not to be, reactivate the resident programs or
drivers one-by-one, starting with the ones you feel are most essential to the
proper operation of your system and test the operation of the system after
rebooting with each new addition. Be patient and systematic.  Once the
specific item which causes the problem is isolated, you can perhaps go on to
resolve the problem fully by doing the following:

 Remove the item permanently~dash~in some cases this might be a little used
utility, which was added at some point in the past and forgotten. If you are
not using it and it is causing a conflict, perhaps the simple solution is to
not load it at all. Since all resident software and drivers require some
memory in which to operate, it makes sense to avoid loading them if they are
not being used.

 Check the documentation for the utility or driver~dash~it may point out
areas of conflict which did not previously exist in your system, but do now.
It may specify command line  parameter switches which may have not been
needed previously, but are now. For example, if the problem seems to be a
conflict between a memory manager, and the use of a network or use of a
special video mode or your graphics card, the problem could be that the
memory manager is mapping memory in the area between 640K and 1 Megabyte that
will be used by the network or video card. By determining where the memory
manager is mapping high memory and referring to the documentation for the
network or video card to see what high memory addresses they use, adding the
correct parameters to the memory manager driver to exclude the mapping of the
areas of conflict can be enough to resolve the problem.

 Try loading the drivers in a different order~dash~in some cases, the order
in which resident programs or drivers are loaded can be important.  

 Contact the developer of the resident program or driver. If you are
encountering a problem, in many cases you will not be alone. A problem or
conflict that may seem quite difficult to you may be commonplace to the
developers of the program and they will be able to assist you.

 Contact Quarterdeck Technical Support if the problem seems to revolve around
one of Quarterdeck's products and you are unable to resolve the problem using
the foregoing technique. You should not feel that these efforts have been in
vain. Be sure to document the findings of your investigation while they are
fresh in your mind. The information you have gained will be of much use in
helping our personnel to further analyze and hopefully, resolve your problem.

~Subhead~ Contacting Quarterdeck Technical Support

If you are contacting us by mail, fax or via information systems, please
include in your correspondence:

~Item~ The version number and serial number of the software product or
products you are inquiring about.

~Item~  A printout from Manifest. If you like, you can select "All Manifest"
from the "What to Print" part of Manifest's print menu, and send the entire
printout to us. If you are sending by FAX or want to save paper by printing
out only specific sections, you can run the TECHSUP.BAT file which came with
Manifest, and send us a printed copy of the TECHSUP.MFT file created.

~Item~ If the problem is that you cannot run Manifest at all, then you
cannot, of course, send us a print out from Manifest. Instead, print out your
CONFIG.SYS and AUTOEXEC.BAT files, and write down what hardware (including
make and model) and software (including version) you are using. The "Problem
Report Card" included with your Quarterdeck product will serve as a template
for the information we consider essential.

~Item~ A detailed description of the problem you are encountering. Please
describe EXACTLY what happens, and the exact text of any error messages that
might be encountered. Describe in detail the results of your troubleshooting
efforts using the method described to create a pure environment as outlined
at the beginning of this Appendix.

~Item~ Naturally, give us information on how to respond to you via mail, fax
or one of the other methods we support. See your Quarterdeck Passport Support
booklet for information on the various avenues for contacting us.

If you are contacting us by telephone:

~Item~ Please prepare yourself by following the troubleshooting procedure and
collecting the same information you would prepare if you were mailing your
request for support. 

~Item~ However when you connect with our technical support representative,
you need only give your serial number and a brief description of your
hardware, software and the problem you are encountering. If the support
technician requires additional information, he will ask you for specific
details. As indicated earlier in the troubleshooting section, there are some
problems which become very familiar to technical support and in some cases,
only a brief description of the system and the problem is all that is needed.
