Chapter 6

Expanded Memory

This chapter covers the expanded memory category of Manifest.
Expanded memory is a major PC innovation, responsible for making
it possible to bring  OS/2-like power to your current DOS-based
system. Expanded memory enables your applications to break
through the DOS 640K barrier to satisfy their data needs.
Expanded memory also breaks through the DOS 640K barrier to run
multiple DOS programs simultaneously (multitask).

Expanded memory might seem confusing. There are a number of
expanded memory specifications, and it has been difficult to
determine the differences (and benefits) which distinguish one
specification from the other.

The need for expanded memory arose from the limitation of the
8088/8086 processors' addressing capabilities. The 8088/8086
processor has one megabyte (1024K) of address space. DOS, the
operating system for the 8088/8086 processor, uses the first 640K
of addresses for running programs; it reserves the remaining 384K
for system use. Although the 80286 and 80386 processors have
greater addressing capabilities, each has a mode compatible with
that of the 8088/8086 processor. This mode is called real mode.
It is real mode that is being used by DOS to run your programs.
Thus, the 80286 and 80386 processors, when running DOS, share in
the 640K memory limitation of programs ~dash~ regardless of the
processor's other capabilities. Over the years, as programs
gained more features, this 640K limit has become a barrier that
more and more application programs have had to contend with.

~Subhead~ Expanded Memory Specifications

Lotus Development Corporation was the primary impetus behind the
emergence of expanded memory as a solution to the 640K
restriction on the size of programs. In 1984 users of 1-2-3
Release 1A were creating spreadsheets whose data was filling 640K
of memory. Because 1-2-3 Release 2, then in development, required
more memory for its code than did Release 1A, the possibility
existed that large spreadsheet users would not be able to take
advantage of Release 2 ~dash~ owing to the lack of sufficient
memory for large, existing spreadsheets. 

Therefore, in conjunction with Intel and later Microsoft, Lotus
developed an expanded memory specification, known as the Lotus-
Intel-Microsoft Expanded Memory Specification (EMS) Version 3.2.
Its purpose was to specify how add-on memory under the control of
a memory manager could be used to permit a program to access a
large amount of program data ~dash~ unrestricted by the 640K
limit. The specification worked within the constraint of the one
megabyte address space of the 8088/8086 processor.

The specification defined a means of making up to eight megabytes
of add-on memory (called expanded memory) available to programs
through the controlled allocation and management of relatively
small areas (pages) of memory. At any one time, programs could
access 64K of expanded memory (in four contiguous 16K pages).
This 64K area is called a page frame. Under program control and
with the aid of a memory manager, these pages could be rapidly
brought into and out of the address space of the processor. The
result was that, for the user of an expanded memory program, the
data storage capacity of the program was now virtually unlimited.
A side benefit was that small programs, under 64K in size, could
actually run in expanded memory.

AST enhanced the EMS 3.2 specification by providing programs
direct access to up to one megabyte of expanded memory at a time
(up from 64K) for either data or code, and by providing the
ability to fill out and/or backfill conventional memory to
provide that direct access. These enhancements enabled large
programs to run in expanded memory. This specification, which was
fully compatible with EMS 3.2, is known as the AST-Quadram-
Ashton-Tate Enhanced Expanded Memory Specification 3.2 (EEMS). 

In 1986, this specification and AST's expanded memory hardware
implementation enabled Quarterdeck's DESQview to run multiple,
large DOS programs simultaneously (multitasking) in EEMS memory.
Thus another DOS 640K barrier was broken. For with EEMS, there
was now a way to do true multitasking of full-size applications
on 8088/8086 and 80286 processors running DOS.

In 1987 Lotus, Intel, and Microsoft expanded upon EMS 3.2 and
added a specification for providing programs direct access to up
to one megabyte of memory, and for providing the ability to fill
out or backfill conventional memory. The new specification also
increased the total memory supported (to 32 megabytes) and added
several additional memory management functions. This
specification is known as the Lotus-Intel-Microsoft Expanded
Memory Specification (EMS) 4.0.

Unfortunately, there continues to be confusion about expanded
memory capabilities of different products on the market today.
The reason is that expanded memory hardware and memory manager
software can be legitimately compliant with EMS 4.0, but can
differ in the functionality and services they provide. The table
at the end of this section will provide a checklist to aid you in
selecting expanded memory hardware and software that best suits
your needs.

~Subhead~ The Expanded Memory Technology

 Think of expanded memory as memory which exists but which has no
location, (i.e., it is not addressable). Whether you have 512K or
8 megabytes of expanded memory, whether you have one board or
several, it cannot be used without the means (a memory manager)
to allocate and to position it into the address space of your
system. For 8088/8086 machines, and 80286 or 80386 machines in
real mode, this address space is the first megabyte (0-1024K) of
memory. When so positioned, it is available to and usable by any
program as code or data, just as if it had been physically
installed on the motherboard at the factory. 

Although this is a brief description, there are several
implications of interest. First, your application programs need
not be aware of the presence of expanded memory for you to enjoy
its benefits. Second, DOS need not be aware of the presence of
expanded memory. In fact, no DOS prior to version 4, is aware of
it. Third, nothing has been said so far about where, within that
first megabyte of address space, expanded memory may be located,
or how much memory may be so located.

The first two points show how easily an expanded memory
enhancement can be integrated into an existing system. While
clearly, some programs must be aware of the presence of expanded
memory to make it useful, none should be disturbed by its
presence, not even DOS. The remaining point flags several issues
addressed by each expanded memory specification.  

There is one fundamental concept shared by all specifications and
their implementations ~dash~ mapping. Mapping is the process
whereby a certain address range in the first megabyte of memory
is supplied with memory physically resident on an expanded memory
board. That address range, because it is addressable by the
processor, and supplied with memory from the expanded memory
board is now usable; it may be read from and written to.

This address range may be re-mapped. It can be filled with memory
from a different area of the total pool of available expanded
memory. This memory can be used, and then again replaced by
another mapping operation.  Through the successive application of
memory allocation, mapping and re-mapping of previously used
memory, this one address range, however large or small, has the
effective capacity of a much larger address space.

It is this aspect of expanded memory which enables users of Lotus
Development's 1-2-3 Release 2 to create spreadsheets larger than
640K. It enables disk caching programs to improve disk I/O
performance with minimal use of conventional memory. It is also a
key factor in DESQview's multitasking capability.

But there are other issues which need to be evaluated when
purchasing expanded memory products. These include: 

~Item~ the number of separate address ranges that the hardware
supports for mapping memory,

~Item~ the size that a mappable address range can be,

~Item~ whether expanded memory can be used to backfill
conventional memory,

~Item~ which of the above features the expanded memory software
supports and

~Item~ how quickly expanded memory mapping operations can be
performed.


There are different expanded memory solutions depending upon your
processor and system. 

~Item~ If you have an 8088/8086 or 80286 PC, these functions are
delivered by an EMS or EEMS expanded memory board and manager.

~Item~ If you have a PS/2 Model 50 or 60 with an IBM 80286 Memory
Expansion Option or an IBM 80286 Expanded Memory Adapter/A or
compatible, you need only a software memory manager, such as
Quarterdeck's QEMM-50/60 ~dash~ because the memory board includes
the necessary expanded memory hardware. 

~Item~ If you have an 80386 PC or PS/2, again you need only a
memory manager, such as Quarterdeck's QEMM-386 ~dash~ because the
processor itself includes the logic necessary to support the
features covered by all the expanded memory specifications.

It should also be pointed out that LIMulator software, which
makes extended memory available as expanded memory, lacks the
hardware assist necessary to support any EMS mapping. It is
therefore not usable for multitasking or for mapping expanded
memory into high memory. 


~Subhead~ EMS Comparison Chart

The following EMS Comparison chart outlines important expanded
memory features and the specifications/hardware devices in which
they are supported. Each row in the chart has an example program
in parenthesis which uses the feature. The chart consists of
seven columns. These columns are defined as follows: 

~Item~ EMS 3.2 Hardware (Only 64K mapping): Memory hardware
capable of handling only the functions required for EMS 3.2. This
involves having only four, 16K contiguous pages of memory mapped
into the 640K-1024K range of addresses.

~Item~ EEMS Hardware (Enhanced Expanded - with 1 Alternate Map):
Memory hardware capable of handling EEMS functions. This includes
mapping multiple areas both above 640K and below 640K in any
available size. There may be several, non-contiguous areas. One
alternate map is available for fast changing to another mapping
set.

~Item~ EMS 4.0 Hardware (Larger than 64K Mapping Hardware without
Alternate Maps): Memory hardware capable of handling more than
64K between 640K-1024K, and also capable of backfilling and
mapping memory below 640K. No hardware alternate maps are
available, so large amounts of memory must be mapped one page at
a time.

~Item~ EMS 4.0 Hardware (Larger than 64K Mapping Hardware with
Alternate Maps): Memory hardware capable of handling more than
64K between 640K-1024K, and also capable of backfilling and
mapping memory below 640K. Hardware alternate maps are available,
so large amounts of memory may be mapped very quickly without the
need to individually map each page. The number of alternate maps
may vary from 1 to 255.

~Item~ EMS 3.2 Software: A software driver capable of performing
functions for EMS 3.2. This includes accessing up to 8MB of
memory using a 64K page frame. EMS 3.2 does not support mapping
memory below 640K, has no capability of using more than 64K at a
time between 640K-1024K, and has no alternate maps.

~Item~ EEMS Software: A software driver capable of performing
functions for EEMS. This includes accessing up to 8MB of memory
using a 64K page frame and other areas between 640K-1024K. It
also includes mapping memory below 640K if the expanded memory
board supplies memory in this area.

~Item~ EMS 4.0 Software: A software driver capable of performing
functions for EMS 4.0. This includes accessing up to 32MB of
memory using a 64K page frame, and possibly other larger areas as
well. This might include mapping backfilled memory below 640K and
mapping multiple areas between 640K-1024K. Alternate maps may
also be supported to quickly map many pages (even discontiguous
ones) at one time. Other software features which are available
include functions to make programming for expanded memory easier
than EMS 3.2. The extra hardware features are only available if
the hardware which the driver uses is capable of supporting the
extra functions. 

~Item~ 80386 PC: Hardware which has an 80386 or 80386SX processor
on it. These processors have memory mapping hardware built into
them, which is used by special software, such as QEMM-386.

The minimum hardware to use EMS 3.2 is the same as for EMS 4.0,
so an EMS 4.0 driver can function on a memory card which was
designed for EMS 3.2. The extra EMS 4.0 features are not
available in this case however.

~Begin Table~ 

DESIRABLE FEATURE       

Store Data in expanded memory (1-2-3 Rel2)

EMS 3.2 HW, EMS 3.2 SW                 X
EMS 3.2 HW, EMS 4 SW                   X
EEMS HW (1 Alternate Map), EEMS SW     X
EEMS HW (1 Alternate Map), EMS 4 SW    X
EMS 4 HW (No Alternate Maps), EMS 4 SW X
EMS 4  HW (Alternate Maps), EMS 4 SW   X
80386 PC QEMM-386                     

Run a <64k program in expanded memory (disk cache)

EMS 3.2 HW, EMS 3.2 SW                 X
EMS 3.2 HW, EMS 4 SW                   X
EEMS HW (1 Alternate Map), EEMS SW     X
EEMS HW (1 Alternate Map), EMS 4 SW    X
EMS 4 HW (No Alternate Maps), EMS 4 SW X
EMS 4  HW (Alternate Maps), EMS 4 SW   X
80386 PC QEMM-386                      X

Support EMS 4.0 software programming features

EMS 3.2 HW, EMS 3.2 SW                
EMS 3.2 HW, EMS 4 SW                   X
EEMS HW (1 Alternate Map), EEMS SW    
EEMS HW (1 Alternate Map), EMS 4 SW    X
EMS 4 HW (No Alternate Maps), EMS 4 SW X
EMS 4  HW (Alternate Maps), EMS 4 SW   X
80386 PC QEMM-386                      X

Map more than 2 contiguous areas of memory (Paradox 3)

EMS 3.2 HW, EMS 3.2 SW                
EMS 3.2 HW, EMS 4 SW                  
EEMS HW (1 Alternate Map), EEMS SW     X
EEMS HW (1 Alternate Map), EMS 4 SW    X
EMS 4 HW (No Alternate Maps), EMS 4 SW X
EMS 4  HW (Alternate Maps), EMS 4 SW   X
80386 PC QEMM-386                      X

Multitask several Windows programs in expanded memory (Windows,,
DESQview)

EMS 3.2 HW, EMS 3.2 SW                
EMS 3.2 HW, EMS 4 SW                  
EEMS HW (1 Alternate Map), EEMS SW     X
EEMS HW (1 Alternate Map), EMS 4 SW    X
EMS 4 HW (No Alternate Maps), EMS 4 SW X
EMS 4  HW (Alternate Maps), EMS 4 SW   X
80386 PC QEMM-386                      X

Multitask several programs in expanded memory (DESQview)

EMS 3.2 HW, EMS 3.2 SW                
EMS 3.2 HW, EMS 4 SW                  
EEMS HW (1 Alternate Map), EEMS SW     X
EEMS HW (1 Alternate Map), EMS 4 SW    X
EMS 4 HW (No Alternate Maps), EMS 4 SW X
EMS 4  HW (Alternate Maps), EMS 4 SW   X
80386 PC QEMM-386                      X 

Multitask several  programs (one high speed communications 
program) in expanded memory (DESQview)

EMS 3.2 HW, EMS 3.2 SW                
EMS 3.2 HW, EMS 4 SW                  
EEMS HW (1 Alternate Map), EEMS SW     X
EEMS HW (1 Alternate Map), EMS 4 SW    X
EMS 4 HW (No Alternate Maps), EMS 4 SW
EMS 4  HW (Alternate Maps), EMS 4 SW   X
80386 PC QEMM-386                      X

Multitask multiple DOS programs (including several high speed
communications programs) in expanded memory (DESQview)

EMS 3.2 HW, EMS 3.2 SW                
EMS 3.2 HW, EMS 4 SW                  
EEMS HW (1 Alternate Map), EEMS SW    
EEMS HW (1 Alternate Map), EMS 4 SW   
EMS 4 HW (No Alternate Maps), EMS 4 SW
EMS 4  HW (Alternate Maps), EMS 4 SW   X
80386 PC QEMM-386                      X

Map high memory (16K areas) to run TSRs and device drivers.
(QEMM,, QRAM)

EMS 3.2 HW, EMS 3.2 SW                
EMS 3.2 HW, EMS 4 SW                  
EEMS HW (1 Alternate Map), EEMS SW     X
EEMS HW (1 Alternate Map), EMS 4 SW    X
EMS 4 HW (No Alternate Maps), EMS 4 SW X
EMS 4  HW (Alternate Maps), EMS 4 SW   X
80386 PC QEMM-386                      X 

Map high memory (4K areas) to run TSRs and device drivers. (QEMM)

EMS 3.2 HW, EMS 3.2 SW                
EMS 3.2 HW, EMS 4 SW                  
EEMS HW (1 Alternate Map), EEMS SW    
EEMS HW (1 Alternate Map), EMS 4 SW   
EMS 4 HW (No Alternate Maps), EMS 4 SW
EMS 4  HW (Alternate Maps), EMS 4 SW  
80386 PC QEMM-386                      X

~End Table~ 

 Expanded Memory Capabilities of EMS hardware/software

~Subhead~ Expanded Overview (PO)

 The Expanded Overview shows the type and amount of expanded
memory in your PC. This screen appears only  if you have expanded
memory installed, and an expanded memory manager is loaded and
enabled.

 The Expanded Overview screen has two sections. The top section
gives you information about your EMS driver. The lower section of
the screen accounts for the memory and other resources managed by
the driver.

 You can use this screen to: 

~Item~ quickly identify the version number of your expanded
memory manager,

~Item~ determine what kind of expanded memory manager you have,
and

~Item~ determine how much expanded memory is being used by your
programs.

~Subhead~ Key Terms

 The Expanded Overview screen has several fields which need a
brief explanation.

~Item~ EMS Driver Name & Version: indicates the name and version
number which identify the expanded memory manager installed.
There are memory managers which are not recognizable by Manifest.
The Intel Aboveboard expanded memory manager is one of these. In
these cases the driver name will not be given.

~Item~ EMS Version: indicates the EMS level of support (EMS 3.2,
EMS 4.0 and EEMS 3.2). This is the version reported by your EMS
software driver. 

~Item~ Page Frame: the address of the expanded memory page frame.
The page frame is a mappable area, 64K in size, which is the
place that almost all EMS programs use for mapping. Other
mappable regions are only used by programs written specifically
for EMS 4.0 or EEMS.

~Item~ Number of Mappable Pages: the number of 16K areas of
memory within the first megabyte address space that an expanded
memory manager identifies as capable of being mapped. At least
four contiguous mappable pages are required to support EMS
memory. Many more than four are needed to support multitasking in
expanded memory.

~Item~ Total Expanded Memory: the total amount of expanded memory
in your PC as reported by the expanded memory manager.

~Item~ Available Expanded Memory: the amount of expanded memory
which is currently not being used. Note that for EMS 4.0 drivers,
the part of conventional memory which is mappable (and thus
controllable by the EMS manager) is counted twice ~dash~ once as
conventional memory and once as used expanded memory, contained
in EMS handle 0. This should account for the discrepancy between
the Total and Available columns even when you are not running any
programs which use expanded memory.

~Item~ Total EMS Handles: the number of EMS Handles the expanded
memory manager supports. This number affects the memory manager's
ability to honor requests for expanded memory. Some memory
managers allow this number to be adjusted by a configuration
parameter. 

~Item~ Available EMS Handles: the number currently unallocated.
This number is not reported by EMS 3.2.

~Item~ Total Real Alternate Maps: shows the number of   alternate
maps available. The presence of hardware alternate maps can
improve performance markedly, especially in multitasking
environments. Simulated alternate maps do not provide the same
level of performance. The number of alternate maps supported
depends upon the capabilities of the expanded memory hardware and
the memory management software which supports it. Early EEMS
hardware supported one alternate map. Systems equipped with
QEMM-386 will have 8 alternate maps available by default.

~Subhead~ Please Note

 If Manifest hangs when you select this screen. Manifest makes
expanded memory manager calls to determine the information it
reports. The likely reason for this problem is that there is a
bug in your EMS driver or in a TSR which intercepts and
mishandles an EMS call Manifest makes.

~Subhead~ Expanded Memory Hints

 LIMulator Software & Expanded Memory: On 80386 systems, EMS
drivers such as QEMM-386 convert the computer's extended memory
into expanded memory by using the paging features of the 80386
chip. This technique allows for a full implementation of EMS 4.0
and EEMS features without any additional hardware. There are
other programs which convert extended memory to expanded memory
on 80286 and/or 80386 systems by copying extended memory back and
forth into the conventional memory space. This sort of program,
often called a LIMulator, provides an implementation of EMS which
is too slow for any purpose other than storing data there, and is
thus unacceptable for purposes of multitasking, conventional
memory back filling or video filling real alternate map usage,
and various other features. If you are using a LIMulator,
Manifest will report that your expanded memory is only being
simulated.

 QEMM-386 users can gain 32K of additional expanded memory: If
you add a MAPS=0 parameter to your CONFIG.SYS line that loads
QEMM, your available expanded memory will increase by 32K. Do not
do this if you also use DESQview.

 Alternate Maps & DESQview: The presence of alternate maps
enables DESQview to better manage high speed communications
programs. Without alternate maps, such programs may occasionally
lose characters when running in the background. With additional
alternate maps, multiple high speed communication programs can be
run with little or no degradation in performance. In systems with
only one alternate map available, you should make sure you open
up your communications program first for the best performance.
Additionally, QEMM-386's alternate maps provide DESQview with the
ability to virtualize graphics programs and to implement its
protection features. DESQview uses one real alternate map for
itself plus one for each window you open.

 Mappable Pages & DESQview: If you are viewing this screen while
running DESQview, the number of mappable pages is four  ~dash~ 
for DESQview is controlling the memory. If you want to see the
number of mappable pages your PC is capable of, or how much
conventional memory is backfilled, run Manifest outside of
DESQview.

~Subhead~ Expanded Pages (PA)

 Expanded Pages displays a map of the first megabyte of memory as
it pertains to use by the expanded memory manager. This display
appears only if your computer has expanded memory and an expanded
memory manager is loaded and enabled.

 Each character in the map represents the status of one 16K
memory area, or "page". Each of the 16 rows contains 4
characters, and thus represents 4 pages, a 64K area. Each page is
either mappable (+), Page Frame (F), or unmappable (U).

 The rows of the map are identified by their hexadecimal address,
where the n is to be replaced by the appropriate hexadecimal,
number appearing along the top of the table. 

 The mappable pages summary shows the Memory Area range (in
hexadecimal), the size of the area, and the range of logical
pages assigned to this memory range. The page frame is present to
allow EMS 3.2 capabilities. Other entries may or may not appear
depending upon your EMS setup.

~Subhead~ Key Terms

 Mappable (+) pages are 16K segments of address space in the
first megabyte into which an expanded memory manager can map.
These segments include those areas where physical memory is
backfilled, such as all or part of your conventional memory, as
well as unused segments above conventional memory in the area
nominally devoted to adapter memory.

 If there are any pages listed on this screen that are shown as
Mappable (+) above conventional memory, you could create high RAM
with them with either the RAM switch to Quarterdeck's QEMM or
with Quarterdeck's QRAM. Anywhere that you have created high RAM
will show up as Unmappable (U) here.

 Page Frame (F) is a mappable area, 64K in size, which is the
place that almost all EMS programs use for mapping. Other
mappable regions are only used by programs written specifically
for EMS 4.0 or EEMS.

 Unmappable(U) pages are those 16K segments of address space into
which memory cannot be mapped because: 

~Item~ the segment is below the backfill boundary of the PC, 

~Item~ the segment is in use by an adapter or has been reserved
as high RAM,

~Item~ the EMS board or memory manager cannot map there or has
been told not to map there,

~Item~ a TSR or environment such as DESQview has reserved the
pages for its own use.

 Logical Page is the number assigned by the expanded memory
manager to each mappable page. The page frame will always be
assigned numbers 0 through 3.

~Subhead~ Technical Hints

 A Program or Device Does Not Work: If you are having problems
with a program or device which does not function properly at
specific times, try adding an exclude statement for the expanded
memory manager or changing the addresses that a specific device
uses. Use this screen to determine which areas are controlled by
the memory manager and compare these addresses with those used by
the devices in question.

 It is important that no device be installed in a mappable area.
If it is, the device may fail to function, or programs which use
expanded memory may not work, and/or the computer might lock up.

 Multitasking in Expanded Memory: If you are using DESQview on a
system with EMS, then there are a number of important
considerations that will effect the multitasking capabilities of
your system. The most important consideration is the number of
mappable pages that your expanded memory hardware supports.
Ideally, all of conventional memory should be mappable, and all
of high memory which is not in use should be mappable.
Unfortunately, only some EMS hardware can accomplish this. EMS
3.2 does not support any mapping of conventional memory, and only
supports 64K mappable area (4 pages) in high memory. This
prohibits using expanded memory for multitasking. EMS 4.0
software drivers have been made available for EMS 3.2 hardware
boards, but this in no way affects the inability to multitask on
such boards.

 On 80386 systems, the processor allows all of memory, including
that on the motherboard, to be mapped so that, if your are using
QEMM-386, then you will have no problem mapping conventional
memory. However, EMS 4.0 and EEMS boards can only control (and
therefore map) conventional memory which they are "backfilling".
That is, any part of your conventional memory which is on your
PC's or AT's motherboard prohibits the EMS board from mapping
conventional memory. This is why you will probably want to
disable your motherboard memory as much as you can and "fill in"
conventional memory from memory on your EMS card. 

 Most PCs and ATs can disable their motherboard memory down to
256K, thus allowing the remainder of conventional memory
(typically 384K) to be controlled by the EMS board. This would
allow multiple programs of nearly 384K in size to be multitasking
in DESQview. Note that on PS/2 model 50s and 60s, QEMM-50/60 can
disable the entire motherboard memory and thus map all of
conventional memory, making it possible to multitask multiple
very large programs. However, this does mean that the megabyte on
your motherboard becomes unusable, "lost" memory.

~Subhead~ Expanded Handles (PN)

 Expanded Handles lists the currently allocated EMS handles. The
table lists the handle number, the amount of memory allocated in
K, and, if a program has named the handle, its assigned name.

 This display appears only if you have expanded memory in your
system.

 You may use this list to determine which of your application
programs make use of expanded memory and the amounts of memory
they are using. In some cases (i.e. in DESQview, or by commands
to the application itself), you may be able to limit the amount
of memory the application is able to acquire if its expanded
memory use is excessive for your purposes.

~Subhead~ Key Terms

 Handle: The EMS handle ID. This number may be between 0 and 255
in EMS 4.0, and will be 4 hexadecimal digits in earlier EMS
versions. EMS 4.0 drivers always have a handle 0 which is the
part of conventional memory which is mappable. 

 Size: The amount of memory (expressed in kilobytes) allocated
for the handle.

 Name : The name a program has assigned a particular handle. The
name is optional, and is a feature of EMS 4.0 specification. The
name exists strictly for informational purposes. Currently very
few programs assign handle names. The EMS program supplied with
QEMM and QRAM allows you to name handles.

~Subhead~ Expanded Timings (PM)

 Expanded timings provides memory access times for your expanded
memory. The purpose of this display is twofold.

~Item~ first, to show how fast your expanded memory is relative
to a PC/XT, and

~Item~ second, to show the realtive speeds of different areas of
expanded memory.

~Subhead~ Test Results

 Speed Test Variances: There may be up to a 10% variance in the
speed reported from one test to another.

 Timings & QEMM-386: If the speed of your expanded memory is
faster than the speed of your ROMs (see First Meg Timings), you
can use QEMM's ROM feature to significantly increase the speed of
ROMs in your PC.

~Subhead~ Expanded Benchmark (PB)

 Expanded Benchmark tests your expanded memory manager for speed
in performing EMS calls. The results are measured in microseconds
(1 million microseconds = 1 second), and show the maximum,
minimum and average times required for the given function. 

 This screen only appears if your PC has expanded memory. Note
that the Benchmark requires 64K of expanded memory to run. If
your PC does not have this much available expanded memory, the
Benchmark will not run. As you will notice, this is a dynamic
display. The test results change until a significant sample has
been taken.

 If your expanded memory manager does not support one or more of
these calls, then the data for the corresponding row will be left
blank. This will also occur if you have too little expanded
memory available for the open handle tests.

 You use these benchmark tests to:

~Item~ compare two or more expanded memory managers.

~Item~ determine interrupt latency to determine how quickly
interrupts can be handled.

~Subhead~ Tests & Results

 The following is a brief explanation of the tests. Shorter
timings indicate expanded memory manager software with higher
performance capabilities. The benchmark timings shown on the
Manifest screen are the results of running these tests using
QEMM-386 on a 20 Mhz 80386 PC.

~Item~ Timer Interrupt Latency:  an indicator of how long the
expanded memory manager disables interrupts at any given time. An
acceptable test time is below 500, although the smaller the
better.

~Item~ Map Single Page: the time required for the expanded memory
manager to map a single page of expanded memory.  This number has
great significance, as it is the function which most EMS
applications, such as 1-2-3 release 2, use repeatedly to access
their data.

~Item~ Real Alternate Map Set: the time it takes to switch to an
alternate map. This number has great significance for
multitasking environments, such as DESQview. It indicates the
minimum amount of time it takes to map in a whole new program.
The maximum in this category added to the maximum timer interrupt
latency indicates the "worst case" for running communications
programs in DESQview. If the numbers add up to less than 500, you
should be able to run 9600 baud communications in a DESQview
window.

~Item~ Save All - nn pages: the time required for the expanded
memory manager to save all EMS pages. The number of mappable
pages at the time of the test is shown in place of nn. When
comparing expanded memory managers, you should configure them so
that nn is the same in order to get a fair comparison for this
test and the next two tests.

~Item~ Map Multiple - nn pages: the time required for the
expanded memory manager to map all available EMS pages. The
number available, which is the pages which do not conflict with
Manifest's memory, is shown in place of nn.

~Item~ Restore All - nn pages: the time required for the expanded
memory manager to restore all EMS Pages. The number of mappable
pages at the time of the test is shown in place of nn. The sum of
the times of the Save All and Restore All tests indicates how
long it would take DESQview to map in a whole new program if
there are no real alternate maps available.

~Item~ Save Page Frame: the time required to save the state of
the EMS Page Frame. 

~Item~ Restore Page Frame: the time required to restore the state
of the EMS Page Frame. The sum of the previous two tests
indicates how long it takes a TSR or device driver using expanded
memory to preserve the current page frame mapping around its own
EMS use. Thus, it effects the performance of expanded memory disk
caches and similar programs.

~Item~ Open Handle - 4 pages: the time required to open an EMS
Handle which has 4 EMS pages. Neither this timing nor the next
three timings are very significant, since they are not made very
frequently. However, the calls might possibly generate large
timer interrupt latencies if the expanded memory manager is not
well-written.

~Item~ Close Handle - 4 pages: the time required to close an EMS
Handle, which has 4 EMS pages.

~Item~ Open Handle - 31 pages: the time required to open an EMS
Handle which has 31 EMS pages. This test will only succeed if
there is sufficient expanded memory. 

~Item~ Close Handle - 31 pages: the time required to close an EMS
Handle which has 31 EMS pages.
